const config = require('./config');
const {attach} = require('node-firebird');
const {promisify} = require('util');
const path = require('path');

const fbAttach = promisify(attach);

async function processingCmdCheckUnload(res) {
    // определяем самую старую неотправленную запись
    const options = config.db;
    options.pageSize = 4096;
    const db = await fbAttach(options);
    try {
        const q = promisify(db.execute);
        logger.debug('Определяем дату самого раннего неотправленного чека (check_unload)')
        const row = await q(
            'SELECT '+
            '    first 1 datediff(second, o.op_datetime, current_timestamp) '+
            '    FROM cat_fiscal_data fd '+
            '    inner join cat_operations o on o.operation_id = fd.operation_id '+
            '    WHERE fd.system_type = 1 '+
            '    and fd.status in (1, 2, 4) '+
            '    ORDER BY o.op_datetime '
        );
        let t;
        if (row[0]) {
            t = row[0][0];
        }
        else {
            // записей нет
            t = 0;
        }
        logger.debug('Секунд прошло на настоящий момент: '+t)
        res.write(JSON.stringify(
            {
                "response":"ok",
                "unload_time": t
            }
        ));
    } finally {
        db.detach();
    }
}

async function processingCmd(cmd, res) {
    if (cmd === 'check_unload') {
        return await processingCmdCheckUnload(res);
    }
    else {
        logger.error('Неизвестная команда cmd='+cmd);
        res.write(JSON.stringify(
            {response: 'error', error: 'Неизвестная команда cmd='+cmd}
        ));
    }
}

module.exports.processingCmd = processingCmd;